#include <sys/heap.h>
#include <sys/event.h>
#include <sys/atom.h>
#include <sys/thread.h>

#include "MessageQ.H"

QHANDLE pMessageQueue;

QHANDLE MQ_Allocate(u_char bits){
	
	MessageQueue* que;
	u_char len = 1<<(bits-1);
	
	ASSERT(bits<7);
	ASSERT(bits>0);

	que = (MessageQueue*)(NutHeapAllocClear(sizeof(MessageQueue)+(sizeof(QMSG)*(len))));
	
	if ( ! que )
		return que;

	que->mask = len-1;
	
	//Link into the Global list
	que->p_next = pMessageQueue;
	pMessageQueue = que;
	
	return que;
}

int	MQ_Broadcast(u_char id,int param,void* data){
	
	QHANDLE pCur=pMessageQueue;
	int ret = 0;
	
	while (pCur){
		ret -= MQ_Post(pCur,id,param,data);
		pCur = pCur->p_next;	
	}
	
	return ret;
}

//Sends a message to a Que without waiting
int MQ_Post(QHANDLE q,u_char id,int param,void* data){

	QMSG* cur;
	
	if ( MQ_Full(q) )
		return -1;

	cur = q->que+q->write;
	
	cur->id = id;
	cur->param = param;
	cur->data = data;	
	
	q->write++;
	q->write &= q->mask;	

	NutEventPostAsync(&q->event_wait);
	
	return 0;
}

/*
	Send and Yield to other thread
*/
int	MQ_Send(QHANDLE q,u_char id,int param,void* data){
	
	if ( MQ_Post(q,id,param,data) == 0 ){
		NutThreadYield();
		return 0;
	}
	
	return -1;
}


int	MQ_Full(QHANDLE que){
	if ( ((que->write+1)& que->mask) == que->read ){
		LogError(0,PSTR("QUE FILL!!!"));
		return -1;
	}
		
	return 0;
}


int	MQ_GetMessage(QHANDLE que, QMSG* msg,u_long timeout){

	if ( NutEventWait(&que->event_wait,timeout) )
		return -1;

	//Are there messages in the queue
	ASSERT( que->read != que->write );

	*msg = *(que->que+que->read);

	que->read++;
	que->read&= que->mask;

	//If more messages then we need to Post so we can get the next one
	if ( que->read != que->write )
		NutEventPostAsync(&que->event_wait);
	
	return 0;
}


